package com.flyco.tablayout;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Rect;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.os.Parcelable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.util.AttributeSet;
import android.util.Log;
import android.util.SparseArray;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.OvershootInterpolator;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.flyco.roundview.RoundTextView;
import com.flyco.tablayout.listener.CustomTabEntity;
import com.flyco.tablayout.listener.OnTabSelectListener;
import com.flyco.tablayout.utils.FragmentChangeManager;
import com.flyco.tablayout.utils.UnreadMsgUtils;
import com.nineoldandroids.animation.TypeEvaluator;
import com.nineoldandroids.animation.ValueAnimator;

import java.util.ArrayList;
import java.util.HashMap;

/** 没有继承HorizontalScrollView不能滑动,对于ViewPager无依赖 */
public class CommonTabLayout extends FrameLayout implements ValueAnimator.AnimatorUpdateListener {
    private Context context;
    private ArrayList<CustomTabEntity> tabEntitys = new ArrayList<>();
    private LinearLayout tabsContainer;
    private int currentTab;
    private int lastTab;
    private int tabCount;
    /** 用于绘制显示器 */
    private Rect indicatorRect = new Rect();
    private GradientDrawable indicatorDrawable = new GradientDrawable();

    private Paint rectPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    private Paint dividerPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    private Paint trianglePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    private Path trianglePath = new Path();
    private static final int STYLE_NORMAL = 0;
    private static final int STYLE_TRIANGLE = 1;
    private static final int STYLE_BLOCK = 2;
    private int indicatorStyle = STYLE_NORMAL;

    private float tabPadding;
    private boolean tabSpaceEqual;
    private float tabWidth;

    /** indicator */
    private int indicatorColor;
    private float indicatorHeight;
    private float indicatorWidth;
    private float indicatorCornerRadius;
    private float indicatorMarginLeft;
    private float indicatorMarginTop;
    private float indicatorMarginRight;
    private float indicatorMarginBottom;
    private long indicatorAnimDuration;
    private boolean indicatorAnimEnable;
    private boolean indicatorBounceEnable;
    private float indicatorGravity;

    /** underline */
    private int underlineColor;
    private float underlineHeight;
    private float underlineGravity;

    /** divider */
    private int dividerColor;
    private float dividerWidth;
    private float dividerPadding;

    /** title */
    private float textsize;
    private int textSelectColor;
    private int textUnselectColor;
    private boolean textBold;
    private boolean textAllCaps;

    /** icon */
    private boolean iconVisible;
    private int iconGravity;
    private float iconWidth;
    private float iconHeight;
    private float iconMargin;

    private int h;

    /** anim */
    private ValueAnimator valueAnimator;
    private OvershootInterpolator interpolator = new OvershootInterpolator(1.5f);

    private FragmentChangeManager fragmentChangeManager;

    public CommonTabLayout(Context context) {
        this(context, null, 0);
    }

    public CommonTabLayout(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CommonTabLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        setWillNotDraw(false);//重写onDraw方法,需要调用这个方法来清除flag
        setClipChildren(false);
        setClipToPadding(false);

        this.context = context;
        tabsContainer = new LinearLayout(context);
        addView(tabsContainer);

        obtainAttributes(context, attrs);

        //get layout_height
        String height = attrs.getAttributeValue("http://schemas.android.com/apk/res/android", "layout_height");

        //create ViewPager
        if (height.equals(ViewGroup.LayoutParams.MATCH_PARENT + "")) {
        } else if (height.equals(ViewGroup.LayoutParams.WRAP_CONTENT + "")) {
        } else {
            int[] systemAttrs = {android.R.attr.layout_height};
            TypedArray a = context.obtainStyledAttributes(attrs, systemAttrs);
            h = a.getDimensionPixelSize(0, ViewGroup.LayoutParams.WRAP_CONTENT);
            a.recycle();
        }

        valueAnimator = ValueAnimator.ofObject(new PointEvaluator(), lp, cp);
        valueAnimator.addUpdateListener(this);
    }

    private void obtainAttributes(Context context, AttributeSet attrs) {
        TypedArray ta = context.obtainStyledAttributes(attrs, R.styleable.CommonTabLayout);

        indicatorStyle = ta.getInt(R.styleable.CommonTabLayout_tl_indicator_style, 0);
        indicatorColor = ta.getColor(R.styleable.CommonTabLayout_tl_indicator_color, Color.parseColor(indicatorStyle == STYLE_BLOCK ? "#4B6A87" : "#ffffff"));
        indicatorHeight = ta.getDimension(R.styleable.CommonTabLayout_tl_indicator_height,
                dp2px(indicatorStyle == STYLE_TRIANGLE ? 4 : (indicatorStyle == STYLE_BLOCK ? -1 : 2)));
        indicatorWidth = ta.getDimension(R.styleable.CommonTabLayout_tl_indicator_width, dp2px(indicatorStyle == STYLE_TRIANGLE ? 10 : -1));
        indicatorCornerRadius = ta.getDimension(R.styleable.CommonTabLayout_tl_indicator_corner_radius, dp2px(indicatorStyle == STYLE_BLOCK ? -1 : 0));
        indicatorMarginLeft = ta.getDimension(R.styleable.CommonTabLayout_tl_indicator_margin_left, dp2px(0));
        indicatorMarginTop = ta.getDimension(R.styleable.CommonTabLayout_tl_indicator_margin_top, dp2px(indicatorStyle == STYLE_BLOCK ? 7 : 0));
        indicatorMarginRight = ta.getDimension(R.styleable.CommonTabLayout_tl_indicator_margin_right, dp2px(0));
        indicatorMarginBottom = ta.getDimension(R.styleable.CommonTabLayout_tl_indicator_margin_bottom, dp2px(indicatorStyle == STYLE_BLOCK ? 7 : 0));
        indicatorAnimEnable = ta.getBoolean(R.styleable.CommonTabLayout_tl_indicator_anim_enable, true);
        indicatorBounceEnable = ta.getBoolean(R.styleable.CommonTabLayout_tl_indicator_bounce_enable, true);
        indicatorAnimDuration = ta.getInt(R.styleable.CommonTabLayout_tl_indicator_anim_duration, -1);
        indicatorGravity = ta.getInt(R.styleable.CommonTabLayout_tl_indicator_gravity, Gravity.BOTTOM);

        underlineColor = ta.getColor(R.styleable.CommonTabLayout_tl_underline_color, Color.parseColor("#ffffff"));
        underlineHeight = ta.getDimension(R.styleable.CommonTabLayout_tl_underline_height, dp2px(0));
        underlineGravity = ta.getInt(R.styleable.CommonTabLayout_tl_underline_gravity, Gravity.BOTTOM);

        dividerColor = ta.getColor(R.styleable.CommonTabLayout_tl_divider_color, Color.parseColor("#ffffff"));
        dividerWidth = ta.getDimension(R.styleable.CommonTabLayout_tl_divider_width, dp2px(0));
        dividerPadding = ta.getDimension(R.styleable.CommonTabLayout_tl_divider_padding, dp2px(12));

        textsize = ta.getDimension(R.styleable.CommonTabLayout_tl_textsize, sp2px(13f));
        textSelectColor = ta.getColor(R.styleable.CommonTabLayout_tl_textSelectColor, Color.parseColor("#ffffff"));
        textUnselectColor = ta.getColor(R.styleable.CommonTabLayout_tl_textUnselectColor, Color.parseColor("#AAffffff"));
        textBold = ta.getBoolean(R.styleable.CommonTabLayout_tl_textBold, false);
        textAllCaps = ta.getBoolean(R.styleable.CommonTabLayout_tl_textAllCaps, false);

        iconVisible = ta.getBoolean(R.styleable.CommonTabLayout_tl_iconVisible, true);
        iconGravity = ta.getInt(R.styleable.CommonTabLayout_tl_iconGravity, Gravity.TOP);
        iconWidth = ta.getDimension(R.styleable.CommonTabLayout_tl_iconWidth, dp2px(0));
        iconHeight = ta.getDimension(R.styleable.CommonTabLayout_tl_iconHeight, dp2px(0));
        iconMargin = ta.getDimension(R.styleable.CommonTabLayout_tl_iconMargin, dp2px(2.5f));

        tabSpaceEqual = ta.getBoolean(R.styleable.CommonTabLayout_tl_tab_space_equal, true);
        tabWidth = ta.getDimension(R.styleable.CommonTabLayout_tl_tab_width, dp2px(-1));
        tabPadding = ta.getDimension(R.styleable.CommonTabLayout_tl_tab_padding, tabSpaceEqual || tabWidth > 0 ? dp2px(0) : dp2px(10));

        ta.recycle();
    }

    public void setTabData(ArrayList<CustomTabEntity> tabEntitys) {
        if (tabEntitys == null || tabEntitys.size() == 0) {
            throw new IllegalStateException("TabEntitys can not be NULL or EMPTY !");
        }

        this.tabEntitys.clear();
        this.tabEntitys.addAll(tabEntitys);

        notifyDataSetChanged();
    }

    /** 关联数据支持同时切换fragments */
    public void setTabData(ArrayList<CustomTabEntity> tabEntitys, FragmentManager fm, int containerViewId, ArrayList<Fragment> fragments) {
        fragmentChangeManager = new FragmentChangeManager(fm, containerViewId, fragments);
        setTabData(tabEntitys);
    }

    /** 更新数据 */
    public void notifyDataSetChanged() {
        tabsContainer.removeAllViews();
        this.tabCount = tabEntitys.size();
        View tabView;
        for (int i = 0; i < tabCount; i++) {
            if (iconGravity == Gravity.LEFT) {
                tabView = View.inflate(context, R.layout.layout_tab_left, null);
            } else if (iconGravity == Gravity.RIGHT) {
                tabView = View.inflate(context, R.layout.layout_tab_right, null);
            } else if (iconGravity == Gravity.BOTTOM) {
                tabView = View.inflate(context, R.layout.layout_tab_bottom, null);
            } else {
                tabView = View.inflate(context, R.layout.layout_tab_top, null);
            }

            tabView.setTag(i);
            addTab(i, tabView);
        }

        updateTabStyles();
    }

    /** 创建并添加tab */
    private void addTab(final int position, View tabView) {
        TextView tv_tab_title = (TextView) tabView.findViewById(R.id.tv_tab_title);
        tv_tab_title.setText(tabEntitys.get(position).getTabTitle());
        ImageView iv_tab_icon = (ImageView) tabView.findViewById(R.id.iv_tab_icon);
        iv_tab_icon.setImageResource(tabEntitys.get(position).getTabUnselectedIcon());

        tabView.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                int position = (Integer) v.getTag();
                if (currentTab != position) {
                    setCurrentTab(position);
                    if (listener != null) {
                        listener.onTabSelect(position);
                    }
                } else {
                    if (listener != null) {
                        listener.onTabReselect(position);
                    }
                }
            }
        });

        /** 每一个Tab的布局参数 */
        LinearLayout.LayoutParams lp_tab = tabSpaceEqual ?
                new LinearLayout.LayoutParams(0, LayoutParams.MATCH_PARENT, 1.0f) :
                new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.MATCH_PARENT);
        if (tabWidth > 0) {
            lp_tab = new LinearLayout.LayoutParams((int) tabWidth, LayoutParams.MATCH_PARENT);
        }
        tabsContainer.addView(tabView, position, lp_tab);
    }

    private void updateTabStyles() {
        for (int i = 0; i < tabCount; i++) {
            View tabView = tabsContainer.getChildAt(i);
            tabView.setPadding((int) tabPadding, 0, (int) tabPadding, 0);
            TextView tv_tab_title = (TextView) tabView.findViewById(R.id.tv_tab_title);
            tv_tab_title.setTextColor(i == currentTab ? textSelectColor : textUnselectColor);
            tv_tab_title.setTextSize(TypedValue.COMPLEX_UNIT_PX, textsize);
//            tv_tab_title.setPadding((int) tabPadding, 0, (int) tabPadding, 0);
            if (textAllCaps) {
                tv_tab_title.setText(tv_tab_title.getText().toString().toUpperCase());
            }

            if (textBold) {
                tv_tab_title.getPaint().setFakeBoldText(textBold);
            }

            ImageView iv_tab_icon = (ImageView) tabView.findViewById(R.id.iv_tab_icon);
            if (iconVisible) {
                iv_tab_icon.setVisibility(View.VISIBLE);
                CustomTabEntity tabEntity = tabEntitys.get(i);
                iv_tab_icon.setImageResource(i == currentTab ? tabEntity.getTabSelectedIcon() : tabEntity.getTabUnselectedIcon());
                LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(
                        iconWidth <= 0 ? LinearLayout.LayoutParams.WRAP_CONTENT : (int) iconWidth,
                        iconHeight <= 0 ? LinearLayout.LayoutParams.WRAP_CONTENT : (int) iconHeight);
                if (iconGravity == Gravity.LEFT) {
                    lp.rightMargin = (int) iconMargin;
                } else if (iconGravity == Gravity.RIGHT) {
                    lp.leftMargin = (int) iconMargin;
                } else if (iconGravity == Gravity.BOTTOM) {
                    lp.topMargin = (int) iconMargin;
                } else {
                    lp.bottomMargin = (int) iconMargin;
                }

                iv_tab_icon.setLayoutParams(lp);
            } else {
                iv_tab_icon.setVisibility(View.GONE);
            }
        }
    }

    private void updateTabSelection(int position) {
        for (int i = 0; i < tabCount; ++i) {
            View tabView = tabsContainer.getChildAt(i);
            final boolean isSelect = i == position;
            TextView tab_title = (TextView) tabView.findViewById(R.id.tv_tab_title);
            tab_title.setTextColor(isSelect ? textSelectColor : textUnselectColor);
            ImageView iv_tab_icon = (ImageView) tabView.findViewById(R.id.iv_tab_icon);
            CustomTabEntity tabEntity = tabEntitys.get(i);
            iv_tab_icon.setImageResource(isSelect ? tabEntity.getTabSelectedIcon() : tabEntity.getTabUnselectedIcon());
        }
    }

    private void calcOffset() {
        final View currentTabView = tabsContainer.getChildAt(this.currentTab);
        cp.left = currentTabView.getLeft();
        cp.right = currentTabView.getRight();

        final View lastTabView = tabsContainer.getChildAt(this.lastTab);
        lp.left = lastTabView.getLeft();
        lp.right = lastTabView.getRight();

        valueAnimator.setObjectValues(lp, cp);
        if (indicatorBounceEnable) {
            valueAnimator.setInterpolator(interpolator);
        }

        if (indicatorAnimDuration < 0) {
            indicatorAnimDuration = indicatorBounceEnable ? 500 : 250;
        }
        valueAnimator.setDuration(indicatorAnimDuration);
        valueAnimator.start();
    }

    private void calcIndicatorRect() {
        View currentTabView = tabsContainer.getChildAt(this.currentTab);
        float left = currentTabView.getLeft();
        float right = currentTabView.getRight();

        indicatorRect.left = (int) left;
        indicatorRect.right = (int) right;

        if (indicatorWidth < 0) {   //indicatorWidth小于0时,原jpardogo's PagerSlidingTabStrip

        } else {//indicatorWidth大于0时,圆角矩形以及三角形
            float indicatorLeft = currentTabView.getLeft() + (currentTabView.getWidth() - indicatorWidth) / 2;

            indicatorRect.left = (int) indicatorLeft;
            indicatorRect.right = (int) (indicatorRect.left + indicatorWidth);
        }
    }

    @Override
    public void onAnimationUpdate(ValueAnimator animation) {
        View currentTabView = tabsContainer.getChildAt(this.currentTab);
        IndicatorPoint p = (IndicatorPoint) animation.getAnimatedValue();
        indicatorRect.left = (int) p.left;
        indicatorRect.right = (int) p.right;
        if (indicatorWidth < 0) {   //indicatorWidth小于0时,原jpardogo's PagerSlidingTabStrip

        } else {//indicatorWidth大于0时,圆角矩形以及三角形
            float indicatorLeft = p.left + (currentTabView.getWidth() - indicatorWidth) / 2;

            indicatorRect.left = (int) indicatorLeft;
            indicatorRect.right = (int) (indicatorRect.left + indicatorWidth);
        }
        invalidate();
    }

    private boolean isFirstDraw = true;

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        if (isInEditMode() || tabCount <= 0) {
            return;
        }

        int height = getHeight();
        int paddingLeft = getPaddingLeft();
        // draw divider
        if (dividerWidth > 0) {
            dividerPaint.setStrokeWidth(dividerWidth);
            dividerPaint.setColor(dividerColor);
            for (int i = 0; i < tabCount - 1; i++) {
                View tab = tabsContainer.getChildAt(i);
                canvas.drawLine(tab.getRight(), dividerPadding, tab.getRight(), height - dividerPadding, dividerPaint);
            }
        }

        // draw underline
        if (underlineHeight > 0) {
            rectPaint.setColor(underlineColor);
            if (underlineGravity == Gravity.BOTTOM) {
                canvas.drawRect(paddingLeft, height - underlineHeight, tabsContainer.getWidth() + paddingLeft, height, rectPaint);
            } else {
                canvas.drawRect(paddingLeft, 0, tabsContainer.getWidth() + paddingLeft, underlineHeight, rectPaint);
            }
        }

        //draw indicator line
        if (indicatorAnimEnable) {
            if (isFirstDraw) {
                isFirstDraw = false;
                calcIndicatorRect();
            }
        } else {
            calcIndicatorRect();
        }


        if (indicatorStyle == STYLE_TRIANGLE) {
            if (indicatorHeight > 0) {
                trianglePaint.setColor(indicatorColor);
                trianglePath.reset();
                trianglePath.moveTo(paddingLeft + indicatorRect.left, height);
                trianglePath.lineTo(paddingLeft + indicatorRect.left / 2 + indicatorRect.right / 2, height - indicatorHeight);
                trianglePath.lineTo(paddingLeft + indicatorRect.right, height);
                trianglePath.close();
                canvas.drawPath(trianglePath, trianglePaint);
            }
        } else if (indicatorStyle == STYLE_BLOCK) {
            if (indicatorHeight < 0) {
                indicatorHeight = height - indicatorMarginTop - indicatorMarginBottom;
            } else {

            }

            if (indicatorHeight > 0) {
                if (indicatorCornerRadius < 0 || indicatorCornerRadius > indicatorHeight / 2) {
                    indicatorCornerRadius = indicatorHeight / 2;
                }

                indicatorDrawable.setColor(indicatorColor);
                indicatorDrawable.setBounds(paddingLeft + (int) indicatorMarginLeft + indicatorRect.left,
                        (int) indicatorMarginTop, (int) (paddingLeft + indicatorRect.right - indicatorMarginRight),
                        (int) (indicatorMarginTop + indicatorHeight));
                indicatorDrawable.setCornerRadius(indicatorCornerRadius);
                indicatorDrawable.draw(canvas);
            }
        } else {
               /* rectPaint.setColor(indicatorColor);
                calcIndicatorRect();
                canvas.drawRect(getPaddingLeft() + indicatorRect.left, getHeight() - indicatorHeight,
                        indicatorRect.right + getPaddingLeft(), getHeight(), rectPaint);*/

            if (indicatorHeight > 0) {
                indicatorDrawable.setColor(indicatorColor);
                if (indicatorGravity == Gravity.BOTTOM) {
                    indicatorDrawable.setBounds(paddingLeft + (int) indicatorMarginLeft + indicatorRect.left,
                            height - (int) indicatorHeight - (int) indicatorMarginBottom,
                            paddingLeft + indicatorRect.right - (int) indicatorMarginRight,
                            height - (int) indicatorMarginBottom);
                } else {
                    indicatorDrawable.setBounds(paddingLeft + (int) indicatorMarginLeft + indicatorRect.left,
                            (int) indicatorMarginTop,
                            paddingLeft + indicatorRect.right - (int) indicatorMarginRight,
                            (int) indicatorHeight + (int) indicatorMarginTop);
                }
                indicatorDrawable.setCornerRadius(indicatorCornerRadius);
                indicatorDrawable.draw(canvas);
            }
        }
    }

    //setter and getter
    public void setCurrentTab(int currentTab) {
        lastTab = this.currentTab;
        this.currentTab = currentTab;
        updateTabSelection(currentTab);
        if (fragmentChangeManager != null) {
            fragmentChangeManager.setFragments(currentTab);
        }
        if (indicatorAnimEnable) {
            calcOffset();
        } else {
            invalidate();
        }
    }

    public void setIndicatorStyle(int indicatorStyle) {
        this.indicatorStyle = indicatorStyle;
        invalidate();
    }

    public void setTabPadding(float tabPadding) {
        this.tabPadding = dp2px(tabPadding);
        updateTabStyles();
    }

    public void setTabSpaceEqual(boolean tabSpaceEqual) {
        this.tabSpaceEqual = tabSpaceEqual;
        updateTabStyles();
    }

    public void setTabWidth(float tabWidth) {
        this.tabWidth = dp2px(tabWidth);
        updateTabStyles();
    }

    public void setIndicatorColor(int indicatorColor) {
        this.indicatorColor = indicatorColor;
        invalidate();
    }

    public void setIndicatorHeight(float indicatorHeight) {
        this.indicatorHeight = dp2px(indicatorHeight);
        invalidate();
    }

    public void setIndicatorWidth(float indicatorWidth) {
        this.indicatorWidth = dp2px(indicatorWidth);
        invalidate();
    }

    public void setIndicatorCornerRadius(float indicatorCornerRadius) {
        this.indicatorCornerRadius = dp2px(indicatorCornerRadius);
        invalidate();
    }

    public void setUnderlineColor(int underlineColor) {
        this.underlineColor = underlineColor;
        invalidate();
    }

    public void setUnderlineHeight(float underlineHeight) {
        this.underlineHeight = dp2px(underlineHeight);
        invalidate();
    }

    public void setIndicatorMargin(float indicatorMarginLeft, float indicatorMarginTop,
                                   float indicatorMarginRight, float indicatorMarginBottom) {
        this.indicatorMarginLeft = dp2px(indicatorMarginLeft);
        this.indicatorMarginTop = dp2px(indicatorMarginTop);
        this.indicatorMarginRight = dp2px(indicatorMarginRight);
        this.indicatorMarginBottom = dp2px(indicatorMarginBottom);
        invalidate();
    }

    public void setIndicatorAnimDuration(long indicatorAnimDuration) {
        this.indicatorAnimDuration = indicatorAnimDuration;
    }

    public void setIndicatorAnimEnable(boolean indicatorAnimEnable) {
        this.indicatorAnimEnable = indicatorAnimEnable;
    }

    public void setIndicatorBounceEnable(boolean indicatorBounceEnable) {
        this.indicatorBounceEnable = indicatorBounceEnable;
    }

    public void setDividerColor(int dividerColor) {
        this.dividerColor = dividerColor;
        invalidate();
    }

    public void setDividerWidth(float dividerWidth) {
        this.dividerWidth = dp2px(dividerWidth);
        invalidate();
    }

    public void setDividerPadding(float dividerPadding) {
        this.dividerPadding = dp2px(dividerPadding);
        invalidate();
    }

    public void setTextsize(float textsize) {
        this.textsize = sp2px(textsize);
        updateTabStyles();
    }

    public void setTextSelectColor(int textSelectColor) {
        this.textSelectColor = textSelectColor;
        updateTabStyles();
    }

    public void setTextUnselectColor(int textUnselectColor) {
        this.textUnselectColor = textUnselectColor;
        updateTabStyles();
    }

    public void setTextBold(boolean textBold) {
        this.textBold = textBold;
        updateTabStyles();
    }

    public void setIconVisible(boolean iconVisible) {
        this.iconVisible = iconVisible;
        updateTabStyles();
    }

    public void setIconGravity(int iconGravity) {
        this.iconGravity = iconGravity;
        notifyDataSetChanged();
    }

    public void setIconWidth(float iconWidth) {
        this.iconWidth = dp2px(iconWidth);
        updateTabStyles();
    }

    public void setIconHeight(float iconHeight) {
        this.iconHeight = dp2px(iconHeight);
        updateTabStyles();
    }

    public void setIconMargin(float iconMargin) {
        this.iconMargin = dp2px(iconMargin);
        updateTabStyles();
    }

    public void setTextAllCaps(boolean textAllCaps) {
        this.textAllCaps = textAllCaps;
        updateTabStyles();
    }


    public int getTabCount() {
        return tabCount;
    }

    public int getCurrentTab() {
        return currentTab;
    }

    public int getIndicatorStyle() {
        return indicatorStyle;
    }

    public float getTabPadding() {
        return tabPadding;
    }

    public boolean isTabSpaceEqual() {
        return tabSpaceEqual;
    }

    public float getTabWidth() {
        return tabWidth;
    }

    public int getIndicatorColor() {
        return indicatorColor;
    }

    public float getIndicatorHeight() {
        return indicatorHeight;
    }

    public float getIndicatorWidth() {
        return indicatorWidth;
    }

    public float getIndicatorCornerRadius() {
        return indicatorCornerRadius;
    }

    public float getIndicatorMarginLeft() {
        return indicatorMarginLeft;
    }

    public float getIndicatorMarginTop() {
        return indicatorMarginTop;
    }

    public float getIndicatorMarginRight() {
        return indicatorMarginRight;
    }

    public float getIndicatorMarginBottom() {
        return indicatorMarginBottom;
    }

    public long getIndicatorAnimDuration() {
        return indicatorAnimDuration;
    }

    public boolean isIndicatorAnimEnable() {
        return indicatorAnimEnable;
    }

    public boolean isIndicatorBounceEnable() {
        return indicatorBounceEnable;
    }

    public int getUnderlineColor() {
        return underlineColor;
    }

    public float getUnderlineHeight() {
        return underlineHeight;
    }

    public int getDividerColor() {
        return dividerColor;
    }

    public float getDividerWidth() {
        return dividerWidth;
    }

    public float getDividerPadding() {
        return dividerPadding;
    }

    public float getTextsize() {
        return textsize;
    }

    public int getTextSelectColor() {
        return textSelectColor;
    }

    public int getTextUnselectColor() {
        return textUnselectColor;
    }

    public boolean isTextBold() {
        return textBold;
    }

    public boolean isTextAllCaps() {
        return textAllCaps;
    }

    public int getIconGravity() {
        return iconGravity;
    }

    public float getIconWidth() {
        return iconWidth;
    }

    public float getIconHeight() {
        return iconHeight;
    }

    public float getIconMargin() {
        return iconMargin;
    }

    public boolean isIconVisible() {
        return iconVisible;
    }

    //setter and getter

    // show MsgTipView
    private Paint textPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    private SparseArray<Boolean> initSetMap = new SparseArray<>();

    /**
     * 显示未读消息
     *
     * @param position 显示tab位置
     * @param num      num小于等于0显示红点,num大于0显示数字
     */
    public void showMsg(int position, int num) {
        if (position >= tabCount) {
            position = tabCount - 1;
        }

        View tabView = tabsContainer.getChildAt(position);
        RoundTextView tipView = (RoundTextView) tabView.findViewById(R.id.rtv_msg_tip);
        if (tipView != null) {
            UnreadMsgUtils.show(tipView, num);

            if (initSetMap.get(position) != null && initSetMap.get(position)) {
                return;
            }

            if (!iconVisible) {
                setMsgMargin(position, 2, 2);
            } else {
                setMsgMargin(position, 0,
                        iconGravity == Gravity.LEFT || iconGravity == Gravity.RIGHT ? 4 : 0);
            }

            initSetMap.put(position, true);
        }
    }

    /**
     * 显示未读红点
     *
     * @param position 显示tab位置
     */
    public void showDot(int position) {
        if (position >= tabCount) {
            position = tabCount - 1;
        }
        showMsg(position, 0);
    }

    public void hideMsg(int position) {
        if (position >= tabCount) {
            position = tabCount - 1;
        }

        View tabView = tabsContainer.getChildAt(position);
        RoundTextView tipView = (RoundTextView) tabView.findViewById(R.id.rtv_msg_tip);
        if (tipView != null) {
            tipView.setVisibility(View.GONE);
        }
    }

    /**
     * 设置提示红点偏移,注意
     * 1.控件为固定高度:参照点为tab内容的右上角
     * 2.控件高度不固定(WRAP_CONTENT):参照点为tab内容的右上角,此时高度已是红点的最高显示范围,所以这时bottomPadding其实就是topPadding
     */
    public void setMsgMargin(int position, float leftPadding, float bottomPadding) {
        if (position >= tabCount) {
            position = tabCount - 1;
        }
        View tabView = tabsContainer.getChildAt(position);
        RoundTextView tipView = (RoundTextView) tabView.findViewById(R.id.rtv_msg_tip);
        if (tipView != null) {
            TextView tv_tab_title = (TextView) tabView.findViewById(R.id.tv_tab_title);
            textPaint.setTextSize(textsize);
            float textWidth = textPaint.measureText(tv_tab_title.getText().toString());
            float textHeight = textPaint.descent() - textPaint.ascent();
            MarginLayoutParams lp = (MarginLayoutParams) tipView.getLayoutParams();

            float iconH = iconHeight;
            float margin = 0;
            if (iconVisible) {
                if (iconH <= 0) {
                    iconH = context.getResources().getDrawable(tabEntitys.get(position).getTabSelectedIcon()).getIntrinsicHeight();
                }
                margin = iconMargin;
            }

            if (iconGravity == Gravity.TOP || iconGravity == Gravity.BOTTOM) {
                lp.leftMargin = dp2px(leftPadding);
                lp.topMargin = h > 0 ? (int) (h - textHeight - iconH - margin) / 2 - dp2px(bottomPadding) : dp2px(bottomPadding);
            } else {
                lp.leftMargin = dp2px(leftPadding);
                lp.topMargin = h > 0 ? (int) (h - Math.max(textHeight, iconH)) / 2 - dp2px(bottomPadding) : dp2px(bottomPadding);
            }

            tipView.setLayoutParams(lp);
        }
    }

    /** 当前类只提供了少许设置未读消息属性的方法,可以通过该方法获取RoundTextView对象从而各种设置 */
    public RoundTextView getMsgView(int position) {
        if (position >= tabCount) {
            position = tabCount - 1;
        }
        View tabView = tabsContainer.getChildAt(position);
        RoundTextView tipView = (RoundTextView) tabView.findViewById(R.id.rtv_msg_tip);
        return tipView;
    }

    private OnTabSelectListener listener;

    public void setOnTabSelectListener(OnTabSelectListener listener) {
        this.listener = listener;
    }

    @Override
    protected Parcelable onSaveInstanceState() {
        Bundle bundle = new Bundle();
        bundle.putParcelable("instanceState", super.onSaveInstanceState());
        bundle.putInt("currentTab", currentTab);
        return bundle;
    }

    @Override
    protected void onRestoreInstanceState(Parcelable state) {
        if (state instanceof Bundle) {
            Bundle bundle = (Bundle) state;
            currentTab = bundle.getInt("currentTab");
            state = bundle.getParcelable("instanceState");
            if (currentTab != 0 && tabsContainer.getChildCount() > 0) {
                updateTabSelection(currentTab);
            }
        }
        super.onRestoreInstanceState(state);
    }


    class IndicatorPoint {
        public float left;
        public float right;
    }

    private IndicatorPoint cp = new IndicatorPoint();
    private IndicatorPoint lp = new IndicatorPoint();

    class PointEvaluator implements TypeEvaluator<IndicatorPoint> {
        @Override
        public IndicatorPoint evaluate(float fraction, IndicatorPoint startValue, IndicatorPoint endValue) {
            float left = startValue.left + fraction * (endValue.left - startValue.left);
            float right = startValue.right + fraction * (endValue.right - startValue.right);
            IndicatorPoint point = new IndicatorPoint();
            point.left = left;
            point.right = right;
            return point;
        }
    }


    protected int dp2px(float dp) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dp * scale + 0.5f);
    }

    protected int sp2px(float sp) {
        final float scale = this.context.getResources().getDisplayMetrics().scaledDensity;
        return (int) (sp * scale + 0.5f);
    }

}
